@@ -1,6 +1,7 @@
-/* exm-browse-page.c
+/*
+ * exm-browse-page.c
  *
- * Copyright 2022-2024 Matthew Jakeman <mjakeman26@outlook.co.nz>
+ * Copyright 2022-2025 Matthew Jakeman <mjakeman26@outlook.co.nz>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -48,6 +49,7 @@ struct _ExmBrowsePage
     GCancellable *cancellable;
 
     // Template Widgets
+    AdwStatusPage       *status_page;
     GtkSearchEntry      *search_entry;
     GtkListBox          *search_results;
     GtkStack            *search_stack;
@@ -228,6 +230,19 @@ on_next_page_result (GObject       *source,
 
     if (G_IS_LIST_MODEL (to_append))
     {
+        GtkWidget *child;
+        GtkAdjustment *adjustment = NULL;
+        double scroll_pos = 0.0;
+
+        // Save scrolled window position
+        child = gtk_widget_get_first_child (GTK_WIDGET (self->status_page));
+        if (child != NULL && GTK_IS_SCROLLED_WINDOW (child))
+        {
+            adjustment = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (child));
+            if (adjustment != NULL)
+                scroll_pos = gtk_adjustment_get_value (adjustment);
+        }
+
         n_items = g_list_model_get_n_items (G_LIST_MODEL (to_append));
 
         // Append to list model
@@ -241,6 +256,10 @@ on_next_page_result (GObject       *source,
                gtk_widget_grab_focus (gtk_widget_get_last_child (GTK_WIDGET (self->search_results)));
         }
 
+        // Restore scrolled window position
+        if (adjustment != NULL)
+            gtk_adjustment_set_value (adjustment, scroll_pos);
+
         // Remove unnecessary model
         g_list_store_remove_all (G_LIST_STORE (to_append));
         g_clear_object (&to_append);
@@ -250,7 +269,7 @@ on_next_page_result (GObject       *source,
 }
 
 static void
-on_load_more_results (AdwButtonRow  *row,
+on_load_more_results (AdwButtonRow  *row G_GNUC_UNUSED,
                       ExmBrowsePage *self)
 {
     const char *query;
@@ -398,6 +417,7 @@ exm_browse_page_class_init (ExmBrowsePageClass *klass)
     GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
     gtk_widget_class_set_template_from_resource (widget_class, g_strdup_printf ("%s/exm-browse-page.ui", RESOURCE_PATH));
+    gtk_widget_class_bind_template_child (widget_class, ExmBrowsePage, status_page);
     gtk_widget_class_bind_template_child (widget_class, ExmBrowsePage, search_entry);
     gtk_widget_class_bind_template_child (widget_class, ExmBrowsePage, search_results);
     gtk_widget_class_bind_template_child (widget_class, ExmBrowsePage, search_stack);
