@@ -1,12 +1,32 @@
+/*
+ * exm-installed-page.c
+ *
+ * Copyright 2022-2025 Matthew Jakeman <mjakeman26@outlook.co.nz>
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-or-later
+ */
+
 #include "exm-installed-page.h"
 
-#include "exm-extension-row.h"
-
-#include "local/exm-manager.h"
-#include "exm-enums.h"
-#include "exm-types.h"
-
 #include "exm-config.h"
+#include "exm-enums.h"
+#include "exm-extension-row.h"
+#include "exm-types.h"
+#include "exm-window.h"
+#include "local/exm-manager.h"
 
 #include <glib/gi18n.h>
 
@@ -17,11 +37,13 @@ struct _ExmInstalledPage
     ExmManager *manager;
 
     // Template Widgets
+    GtkStack *stack;
+    AdwBanner *updates_banner;
+    AdwSwitchRow *global_toggle;
     GtkListBox *user_list_box;
     GtkListBox *system_list_box;
-    GtkLabel *num_updates_label;
-    GtkRevealer *updates_action_bar;
-    AdwSwitchRow *global_toggle;
+    GtkListBox *search_list_box;
+    GtkFilterListModel *search_list_model;
 
     gboolean sort_enabled_first;
 };
@@ -101,24 +123,37 @@ exm_installed_page_set_property (GObject      *object,
     }
 }
 
+GtkStack *
+exm_installed_page_get_stack (ExmInstalledPage *self)
+{
+    return self->stack;
+}
+
+GtkFilterListModel *
+exm_installed_page_get_search_list_model (ExmInstalledPage *self)
+{
+    return self->search_list_model;
+}
+
 static GtkWidget *
 widget_factory (ExmExtension     *extension,
                 ExmInstalledPage *self)
 {
     ExmExtensionRow *row;
-
-    g_return_if_fail (EXM_IS_EXTENSION (extension));
-    g_return_if_fail (EXM_IS_INSTALLED_PAGE (self));
+    g_return_val_if_fail (EXM_IS_EXTENSION (extension), GTK_WIDGET (NULL));
+    g_return_val_if_fail (EXM_IS_INSTALLED_PAGE (self), GTK_WIDGET (NULL));
 
     row = exm_extension_row_new (extension, self->manager);
     return GTK_WIDGET (row);
 }
 
 static int
-compare_enabled (ExmExtension *this, ExmExtension *other)
+compare_enabled (ExmExtension *this,
+                 ExmExtension *other,
+                 gpointer      user_data G_GNUC_UNUSED)
 {
-    g_return_if_fail (EXM_IS_EXTENSION (this));
-    g_return_if_fail (EXM_IS_EXTENSION (other));
+    g_return_val_if_fail (EXM_IS_EXTENSION (this), 2);
+    g_return_val_if_fail (EXM_IS_EXTENSION (other), 2); // Crash
 
     ExmExtensionState this_state;
     ExmExtensionState other_state;
@@ -135,26 +170,60 @@ compare_enabled (ExmExtension *this, ExmExtension *other)
         return -1;
     else if (!this_enabled && other_enabled)
         return 1;
+
+    return 0; // Never reached
 }
 
 static void
-bind_list_box (GtkListBox       *list_box,
-               GListModel       *model,
-               gboolean          sort_enabled_first,
+on_search_changed (GtkSearchEntry *search_entry,
+                   gpointer        user_data)
+{
+    GtkStringFilter *search_filter = (GtkStringFilter *)user_data;
+
+    gtk_string_filter_set_search (search_filter, gtk_editable_get_text (GTK_EDITABLE (search_entry)));
+}
+
+static void
+on_visible_stack_changed (GObject    *object G_GNUC_UNUSED,
+                          GParamSpec *pspec G_GNUC_UNUSED,
+                          gpointer    user_data)
+{
+    ExmInstalledPage *self = (ExmInstalledPage *) user_data;
+    ExmWindow *window;
+    gboolean search_mode;
+
+    window = EXM_WINDOW (gtk_widget_get_ancestor (GTK_WIDGET (self), EXM_TYPE_WINDOW));
+    search_mode = gtk_search_bar_get_search_mode (exm_window_get_search_bar (window));
+
+    if (search_mode && g_list_model_get_n_items (G_LIST_MODEL (self->search_list_model)) > 0)
+        gtk_stack_set_visible_child_name (self->stack , "page_results");
+    else if (search_mode)
+        gtk_stack_set_visible_child_name (self->stack , "page_empty");
+    else
+        gtk_stack_set_visible_child_name (self->stack , "page_list");
+}
+
+static void
+bind_list_box (GListModel       *model,
                ExmInstalledPage *self)
 {
     GtkExpression *expression;
     GtkStringSorter *alphabetical_sorter;
     GtkSortListModel *sorted_model;
+    GtkStringFilter *search_filter;
+    GtkBoolFilter *is_user_filter;
+    GtkFilterListModel *filtered_model;
+    ExmWindow *window;
+    GtkSearchEntry *search_entry;
+    const gchar *search_text = NULL;
 
-    g_return_if_fail (GTK_IS_LIST_BOX (list_box));
     g_return_if_fail (G_IS_LIST_MODEL (model));
 
     // Sort alphabetically
     expression = gtk_property_expression_new (EXM_TYPE_EXTENSION, NULL, "display-name");
     alphabetical_sorter = gtk_string_sorter_new (expression);
 
-    if (sort_enabled_first)
+    if (self->sort_enabled_first)
     {
         GtkCustomSorter *enabled_sorter;
         GtkMultiSorter *multi_sorter;
@@ -173,63 +242,90 @@ bind_list_box (GtkListBox       *list_box,
         sorted_model = gtk_sort_list_model_new (model, GTK_SORTER (alphabetical_sorter));
     }
 
-    gtk_list_box_bind_model (list_box, G_LIST_MODEL (sorted_model),
+    search_filter = gtk_string_filter_new (expression);
+    self->search_list_model = gtk_filter_list_model_new (G_LIST_MODEL (sorted_model), GTK_FILTER (search_filter));
+
+    gtk_list_box_bind_model (self->search_list_box, G_LIST_MODEL (self->search_list_model),
                              (GtkListBoxCreateWidgetFunc) widget_factory,
                              self, NULL);
+
+    // Filter by user/system extension
+    expression = gtk_property_expression_new (EXM_TYPE_EXTENSION, NULL, "is-user");
+    is_user_filter = gtk_bool_filter_new (expression);
+    filtered_model = gtk_filter_list_model_new (G_LIST_MODEL (sorted_model), GTK_FILTER (is_user_filter));
+
+    gtk_list_box_bind_model (self->user_list_box, G_LIST_MODEL (filtered_model),
+                             (GtkListBoxCreateWidgetFunc) widget_factory,
+                             self, NULL);
+
+    is_user_filter = gtk_bool_filter_new (expression);
+    gtk_bool_filter_set_invert (is_user_filter, TRUE);
+    filtered_model = gtk_filter_list_model_new (G_LIST_MODEL (sorted_model), GTK_FILTER (is_user_filter));
+
+    gtk_list_box_bind_model (self->system_list_box, G_LIST_MODEL (filtered_model),
+                             (GtkListBoxCreateWidgetFunc) widget_factory,
+                             self, NULL);
+
+    window = EXM_WINDOW (gtk_widget_get_ancestor (GTK_WIDGET (self), EXM_TYPE_WINDOW));
+    search_entry = GTK_SEARCH_ENTRY (gtk_search_bar_get_child (exm_window_get_search_bar (window)));
+    search_text = gtk_editable_get_text (GTK_EDITABLE (search_entry));
+
+    // Refilter when sort-enabled-first changes and there is an ongoing search
+    if (search_text)
+      gtk_string_filter_set_search (search_filter, search_text);
+
+    g_signal_connect (search_entry,
+                      "search-changed",
+                      G_CALLBACK (on_search_changed),
+                      search_filter);
+
+    g_signal_connect (self->search_list_model,
+                      "notify::n-items",
+                      G_CALLBACK (on_visible_stack_changed),
+                      self);
 }
 
 static guint
-show_updates_bar (ExmInstalledPage *self)
+show_updates_banner (ExmInstalledPage *self)
 {
-    gtk_revealer_set_reveal_child (self->updates_action_bar, TRUE);
+    adw_banner_set_revealed (self->updates_banner, TRUE);
 
     return G_SOURCE_REMOVE;
 }
 
 static void
-on_updates_available (ExmManager       *manager,
+on_updates_available (ExmManager       *manager G_GNUC_UNUSED,
                       int               n_updates,
                       ExmInstalledPage *self)
 {
     char *label;
 
     // Translators: '%d' = number of extensions that will be updated
-    label = g_strdup_printf(ngettext("One extension will be updated on next login.",
-                                     "%d extensions will be updated on next login.",
+    label = g_strdup_printf(ngettext("%d extension will be updated on next login",
+                                     "%d extensions will be updated on next login",
                                      n_updates), n_updates);
 
-    gtk_label_set_label (self->num_updates_label, label);
+    adw_banner_set_title (self->updates_banner, label);
     g_free (label);
 
     // Short delay to draw user attention
-    g_timeout_add (500, G_SOURCE_FUNC (show_updates_bar), self);
+    g_timeout_add (500, G_SOURCE_FUNC (show_updates_banner), self);
 }
 
 static void
 invalidate_model_bindings (ExmInstalledPage *self)
 {
-    GListModel *user_ext_model;
-    GListModel *system_ext_model;
+    GListModel *ext_model;
 
     if (!self->manager)
         return;
 
     g_object_get (self->manager,
-                  "user-extensions", &user_ext_model,
-                  "system-extensions", &system_ext_model,
+                  "extensions", &ext_model,
                   NULL);
 
-    if (user_ext_model)
-        bind_list_box (self->user_list_box,
-                       user_ext_model,
-                       self->sort_enabled_first,
-                       self);
-
-    if (system_ext_model)
-        bind_list_box (self->system_list_box,
-                       system_ext_model,
-                       self->sort_enabled_first,
-                       self);
+    if (ext_model)
+        bind_list_box (ext_model, self);
 }
 
 static void
@@ -249,18 +345,6 @@ on_bind_manager (ExmInstalledPage *self)
                             "active",
                             G_BINDING_BIDIRECTIONAL|G_BINDING_SYNC_CREATE);
 
-    g_object_bind_property (self->manager,
-                            "extensions-enabled",
-                            self->user_list_box,
-                            "sensitive",
-                            G_BINDING_SYNC_CREATE);
-
-    g_object_bind_property (self->manager,
-                            "extensions-enabled",
-                            self->system_list_box,
-                            "sensitive",
-                            G_BINDING_SYNC_CREATE);
-
     // Check if updates are available
     // NOTE: We need to do this *after* connecting the signal
     // handler above, otherwise we will not be notified.
@@ -294,12 +378,14 @@ exm_installed_page_class_init (ExmInstalledPageClass *klass)
 
     GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
-    gtk_widget_class_set_template_from_resource (widget_class, "/com/mattjakeman/ExtensionManager/exm-installed-page.ui");
+    gtk_widget_class_set_template_from_resource (widget_class, g_strdup_printf ("%s/exm-installed-page.ui", RESOURCE_PATH));
+
+    gtk_widget_class_bind_template_child (widget_class, ExmInstalledPage, stack);
+    gtk_widget_class_bind_template_child (widget_class, ExmInstalledPage, updates_banner);
+    gtk_widget_class_bind_template_child (widget_class, ExmInstalledPage, global_toggle);
     gtk_widget_class_bind_template_child (widget_class, ExmInstalledPage, user_list_box);
     gtk_widget_class_bind_template_child (widget_class, ExmInstalledPage, system_list_box);
-    gtk_widget_class_bind_template_child (widget_class, ExmInstalledPage, num_updates_label);
-    gtk_widget_class_bind_template_child (widget_class, ExmInstalledPage, updates_action_bar);
-    gtk_widget_class_bind_template_child (widget_class, ExmInstalledPage, global_toggle);
+    gtk_widget_class_bind_template_child (widget_class, ExmInstalledPage, search_list_box);
 
     gtk_widget_class_bind_template_callback (widget_class, on_bind_manager);
 
@@ -323,7 +409,7 @@ create_user_placeholder ()
 
     adw_action_row_add_prefix (ADW_ACTION_ROW (row), icon);
     adw_preferences_row_set_title (ADW_PREFERENCES_ROW (row),
-                                   _("There are no user extensions installed."));
+                                   _("No User Extensions Installed"));
     adw_action_row_add_suffix (ADW_ACTION_ROW (row), button);
 
     return row;
@@ -338,10 +424,9 @@ create_system_placeholder ()
     icon = gtk_image_new_from_icon_name ("settings-symbolic");
     adw_action_row_add_prefix (ADW_ACTION_ROW (row), icon);
     adw_preferences_row_set_title (ADW_PREFERENCES_ROW (row),
-                                   _("There are no system extensions installed."));
+                                   _("No System Extensions Installed"));
 
     return row;
-
 }
 
 static void
@@ -364,6 +449,4 @@ exm_installed_page_init (ExmInstalledPage *self)
 
     gtk_list_box_set_placeholder (self->system_list_box,
                                   create_system_placeholder ());
-
-
 }
