@@ -1,7 +1,7 @@
 /*
  * exm-upgrade-assistant.c
  *
- * Copyright 2022 Matthew Jakeman <mjakeman26@outlook.co.nz>
+ * Copyright 2022-2025 Matthew Jakeman <mjakeman26@outlook.co.nz>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,6 +24,8 @@
 #include "web/exm-data-provider.h"
 #include "exm-upgrade-result.h"
 
+#include "exm-config.h"
+
 #include <glib/gi18n.h>
 
 struct _ExmUpgradeAssistant
@@ -56,7 +58,10 @@ struct _ExmUpgradeAssistant
     GtkLabel *description;
 
     // Waiting Page
-    GtkLabel *counter;
+    AdwStatusPage *counter;
+
+    // Error Page
+    AdwStatusPage *error_status;
 
     // Results Page
     GtkListBox *user_list_box;
@@ -84,14 +89,6 @@ exm_upgrade_assistant_new (ExmManager *manager)
                          NULL);
 }
 
-static void
-exm_upgrade_assistant_finalize (GObject *object)
-{
-    ExmUpgradeAssistant *self = (ExmUpgradeAssistant *)object;
-
-    G_OBJECT_CLASS (exm_upgrade_assistant_parent_class)->finalize (object);
-}
-
 static void
 exm_upgrade_assistant_get_property (GObject    *object,
                                     guint       prop_id,
@@ -177,7 +174,7 @@ update_checked_count (ExmUpgradeAssistant *self)
                             self->number_checked,
                             self->total_extensions);
 
-    gtk_label_set_text (self->counter, text);
+    adw_status_page_set_title (self->counter, text);
     g_free (text);
 }
 
@@ -212,7 +209,7 @@ display_results (ExmUpgradeAssistant *self)
         gtk_widget_add_css_class (GTK_WIDGET (self->progress_bar), "error");
     }
 
-    text = _("<b>GNOME %s</b> supports <b>%d out of %d</b> of the extensions currently installed on the system.");
+    text = _("<b>GNOME %s</b> supports <b>%d out of %d</b> of the extensions currently installed on the system");
     text = g_strdup_printf (text, self->target_shell_version, self->number_supported, self->total_extensions);
     gtk_label_set_markup (self->summary, text);
     g_free (text);
@@ -250,7 +247,7 @@ print_list_model (GListModel  *model,
     num_extensions = g_list_model_get_n_items (model);
     for (i = 0; i < num_extensions; i++) {
         ExmUpgradeResult *result;
-        const gchar *name, *creator, *uuid, *url, *supported_text;
+        const gchar *name, *creator, *uuid, *supported_text;
         SupportStatus supported;
 
         result = g_list_model_get_item (model, i);
@@ -344,15 +341,13 @@ display_extension_result (ExmUpgradeAssistant *self,
                           ExmUpgradeResult    *result,
                           gboolean             is_user)
 {
-    if (get_support_status (result, self->target_shell_version) == STATUS_SUPPORTED) {
+    if (get_support_status (result, self->target_shell_version) == STATUS_SUPPORTED)
         self->number_supported++;
-    }
 
     g_list_store_append (is_user ? self->user_results_store : self->system_results_store, result);
 
-    if (self->waiting_on_tasks && self->number_checked == self->total_extensions) {
+    if (self->waiting_on_tasks && self->number_checked == self->total_extensions)
         display_results (self);
-    }
 }
 
 static void
@@ -362,25 +357,40 @@ on_extension_processed (GObject      *source,
 {
     ExmSearchResult *web_info;
     GError *error = NULL;
-    ExmUpgradeAssistant *self;
     ExtensionCheckData *data;
+    ExmUpgradeAssistant *self;
     ExmUpgradeResult *result;
 
     g_return_if_fail (user_data != NULL);
 
+    web_info = exm_data_provider_get_finish (EXM_DATA_PROVIDER (source), async_result, &error);
     data = (ExtensionCheckData *) user_data;
     self = EXM_UPGRADE_ASSISTANT (data->assistant);
 
+    if (error && error->code != 404)
+    {
+        // Filter 5xx status codes (server errors)
+        if (error->code / 100 == 5)
+            adw_status_page_set_description (self->error_status, _("Check <a href='https://status.gnome.org/'>GNOME infrastructure status</a> and try again later"));
+        else
+            adw_status_page_set_description (self->error_status, _("Check your network status and try again"));
+
+        gtk_stack_set_visible_child_name (self->stack, "error");
+
+        g_clear_error (&error);
+        free_check_data (data);
+
+        return;
+    }
+
     self->number_checked++;
     update_checked_count (self);
 
     result = exm_upgrade_result_new ();
     exm_upgrade_result_set_local_data (result, data->local_data);
 
-    if ((web_info = exm_data_provider_get_finish (EXM_DATA_PROVIDER (source), async_result, &error)) != FALSE)
-    {
+    if (EXM_IS_SEARCH_RESULT (web_info))
         exm_upgrade_result_set_web_data (result, web_info);
-    }
 
     display_extension_result (self, result, data->is_user);
     free_check_data (data);
@@ -389,8 +399,7 @@ on_extension_processed (GObject      *source,
 static void
 do_compatibility_check (ExmUpgradeAssistant *self)
 {
-    GListModel *user_ext_model;
-    GListModel *system_ext_model;
+    GListModel *ext_model;
     int num_items;
     int i;
 
@@ -406,7 +415,7 @@ do_compatibility_check (ExmUpgradeAssistant *self)
     if (!key)
         return;
 
-    target_version = (int) g_hash_table_lookup (self->version_map, key);
+    target_version = GPOINTER_TO_INT (g_hash_table_lookup (self->version_map, key));
 
     if (self->target_shell_version)
         g_free (self->target_shell_version);
@@ -417,8 +426,7 @@ do_compatibility_check (ExmUpgradeAssistant *self)
         return;
 
     g_object_get (self->manager,
-                  "user-extensions", &user_ext_model,
-                  "system-extensions", &system_ext_model,
+                  "extensions", &ext_model,
                   NULL);
 
     // Display spinner
@@ -435,35 +443,22 @@ do_compatibility_check (ExmUpgradeAssistant *self)
     g_list_store_remove_all (self->user_results_store);
     g_list_store_remove_all (self->system_results_store);
 
-    num_items = g_list_model_get_n_items (user_ext_model);
+    num_items = g_list_model_get_n_items (ext_model);
     for (i = 0; i < num_items; i++) {
         char *uuid;
+        gboolean is_user;
         ExmExtension *extension;
         ExtensionCheckData *data;
 
-        extension = EXM_EXTENSION (g_list_model_get_item (user_ext_model, i));
+        extension = EXM_EXTENSION (g_list_model_get_item (ext_model, i));
 
-        g_object_get (extension, "uuid", &uuid, NULL);
+        g_object_get (extension,
+                      "uuid", &uuid,
+                      "is-user", &is_user,
+                      NULL);
         g_debug ("Processing: %s\n", uuid);
 
-        data = create_check_data (extension, self, TRUE);
-
-        self->total_extensions++;
-        exm_data_provider_get_async (self->data_provider, uuid, NULL, on_extension_processed, data);
-    }
-
-    num_items = g_list_model_get_n_items (system_ext_model);
-    for (i = 0; i < num_items; i++) {
-        char *uuid;
-        ExmExtension *extension;
-        ExtensionCheckData *data;
-
-        extension = EXM_EXTENSION (g_list_model_get_item (system_ext_model, i));
-
-        g_object_get (extension, "uuid", &uuid, NULL);
-        g_debug ("Processing: %s\n", uuid);
-
-        data = create_check_data (extension, self, FALSE);
+        data = create_check_data (extension, self, is_user);
 
         self->total_extensions++;
         exm_data_provider_get_async (self->data_provider, uuid, NULL, on_extension_processed, data);
@@ -480,7 +475,7 @@ widget_factory (ExmUpgradeResult    *result,
 {
     SupportStatus supported;
     const gchar *name, *creator;
-    GtkWidget *hbox, *vbox, *label, *status;
+    GtkWidget *row, *status;
 
     g_return_val_if_fail (EXM_IS_UPGRADE_RESULT (result), NULL);
 
@@ -489,50 +484,35 @@ widget_factory (ExmUpgradeResult    *result,
 
     supported = get_support_status (result, self->target_shell_version);
 
-    // TODO: Append icon?
-    hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 4);
-    gtk_widget_add_css_class (hbox, "upgrade-assistant-result");
-
-    vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 4);
-    gtk_widget_set_hexpand (vbox, TRUE);
-    gtk_box_append (GTK_BOX (hbox), vbox);
-
-    label = gtk_label_new (name);
-    gtk_label_set_ellipsize (GTK_LABEL (label), PANGO_ELLIPSIZE_END);
-    gtk_label_set_xalign (GTK_LABEL (label), 0.0f);
-    gtk_widget_add_css_class (label, "heading");
-    gtk_box_append (GTK_BOX (vbox), label);
-
-    label = gtk_label_new (creator);
-    gtk_label_set_xalign (GTK_LABEL (label), 0.0f);
-    gtk_widget_add_css_class (label, "dim-label");
-    gtk_box_append (GTK_BOX (vbox), label);
+    row = adw_action_row_new ();
+    adw_preferences_row_set_title (ADW_PREFERENCES_ROW (row) , name);
+    adw_action_row_set_subtitle (ADW_ACTION_ROW (row), creator);
 
     if (supported == STATUS_SUPPORTED) {
-        status = gtk_label_new (_("Supported"));
+        status = gtk_image_new_from_icon_name ("supported-symbolic");
         gtk_widget_add_css_class (status, "success");
-        gtk_widget_set_tooltip_text (GTK_WIDGET (status), _("A compatible version of the extension exists."));
+        gtk_widget_set_tooltip_text (GTK_WIDGET (status), _("A compatible version of the extension exists"));
     } else if (supported == STATUS_UNSUPPORTED) {
-        status = gtk_label_new (_("Unsupported"));
+        status = gtk_image_new_from_icon_name ("unsupported-symbolic");
         gtk_widget_add_css_class (status, "error");
-        gtk_widget_set_tooltip_text (GTK_WIDGET (status), _("No compatible version of the extension exists."));
+        gtk_widget_set_tooltip_text (GTK_WIDGET (status), _("No compatible version of the extension exists"));
     } else {
-        status = gtk_label_new (_("Unknown"));
+        status = gtk_image_new_from_icon_name ("unknown-symbolic");
         gtk_widget_add_css_class (status, "warning");
         gtk_widget_set_tooltip_text (GTK_WIDGET (status), _("This extension is not hosted on extensions.gnome.org. Its compatibility cannot be determined."));
     }
 
-    gtk_widget_set_valign (status, GTK_ALIGN_CENTER);
-    gtk_widget_set_halign (status, GTK_ALIGN_CENTER);
-    gtk_box_append (GTK_BOX (hbox), status);
+    gtk_image_set_pixel_size (GTK_IMAGE (status), 24);
+    gtk_widget_set_focusable (status, TRUE);
+    adw_action_row_add_suffix (ADW_ACTION_ROW (row), status);
 
-    return GTK_WIDGET (hbox);
+    return row;
 }
 
 static void
 bind_list_box (ExmUpgradeAssistant *self,
-               GtkListBox *list_box,
-               GListModel *model)
+               GtkListBox          *list_box,
+               GListModel          *model)
 {
     GtkExpression *expression;
     GtkStringSorter *alphabetical_sorter;
@@ -581,7 +561,7 @@ populate_drop_down (ExmUpgradeAssistant *self)
     else if (month >= G_DATE_SEPTEMBER)
         current_gnome_version += 1;
 
-    g_print ("Current GNOME Version: %d\n", current_gnome_version);
+    g_info ("Current GNOME Version: %d\n", current_gnome_version);
 
     // Make sure we at least have GNOME 40-43 regardless
     // of the date calculation. Update this periodically
@@ -595,7 +575,7 @@ populate_drop_down (ExmUpgradeAssistant *self)
         gchar *key;
 
         key = g_strdup_printf ("GNOME %d", index);
-        g_hash_table_insert (hash_table, key, index);
+        g_hash_table_insert (hash_table, key, GINT_TO_POINTER (index));
         gtk_string_list_append (string_list, key);
     }
 
@@ -627,7 +607,6 @@ exm_upgrade_assistant_class_init (ExmUpgradeAssistantClass *klass)
 {
     GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-    object_class->finalize = exm_upgrade_assistant_finalize;
     object_class->get_property = exm_upgrade_assistant_get_property;
     object_class->set_property = exm_upgrade_assistant_set_property;
 
@@ -642,7 +621,7 @@ exm_upgrade_assistant_class_init (ExmUpgradeAssistantClass *klass)
 
     GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
-    gtk_widget_class_set_template_from_resource (widget_class, "/com/mattjakeman/ExtensionManager/exm-upgrade-assistant.ui");
+    gtk_widget_class_set_template_from_resource (widget_class, g_strdup_printf ("%s/exm-upgrade-assistant.ui", RESOURCE_PATH));
     gtk_widget_class_bind_template_child (widget_class, ExmUpgradeAssistant, user_list_box);
     gtk_widget_class_bind_template_child (widget_class, ExmUpgradeAssistant, system_list_box);
     gtk_widget_class_bind_template_child (widget_class, ExmUpgradeAssistant, run_button);
@@ -650,6 +629,7 @@ exm_upgrade_assistant_class_init (ExmUpgradeAssistantClass *klass)
     gtk_widget_class_bind_template_child (widget_class, ExmUpgradeAssistant, description);
     gtk_widget_class_bind_template_child (widget_class, ExmUpgradeAssistant, stack);
     gtk_widget_class_bind_template_child (widget_class, ExmUpgradeAssistant, counter);
+    gtk_widget_class_bind_template_child (widget_class, ExmUpgradeAssistant, error_status);
     gtk_widget_class_bind_template_child (widget_class, ExmUpgradeAssistant, progress_bar);
     gtk_widget_class_bind_template_child (widget_class, ExmUpgradeAssistant, summary);
     gtk_widget_class_bind_template_child (widget_class, ExmUpgradeAssistant, copy_details);
@@ -677,14 +657,15 @@ exm_upgrade_assistant_init (ExmUpgradeAssistant *self)
     placeholder = adw_action_row_new ();
     icon = gtk_image_new_from_icon_name ("puzzle-piece-symbolic");
     adw_action_row_add_prefix (ADW_ACTION_ROW (placeholder), icon);
-    adw_preferences_row_set_title (ADW_PREFERENCES_ROW (placeholder), _("There are no user extensions installed."));
+    adw_preferences_row_set_title (ADW_PREFERENCES_ROW (placeholder), _("No User Extensions Installed"));
     gtk_list_box_set_placeholder (self->user_list_box, placeholder);
 
     placeholder = adw_action_row_new ();
     icon = gtk_image_new_from_icon_name ("settings-symbolic");
     adw_action_row_add_prefix (ADW_ACTION_ROW (placeholder), icon);
-    adw_preferences_row_set_title (ADW_PREFERENCES_ROW (placeholder), _("There are no system extensions installed."));
+    adw_preferences_row_set_title (ADW_PREFERENCES_ROW (placeholder), _("No System Extensions Installed"));
     gtk_list_box_set_placeholder (self->system_list_box, placeholder);
 
     populate_drop_down (self);
 }
+
