@@ -1,6 +1,7 @@
-/* exm-window.c
+/*
+ * exm-window.c
  *
- * Copyright 2022 Matthew Jakeman
+ * Copyright 2022-2025 Matthew Jakeman <mjakeman26@outlook.co.nz>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,19 +15,21 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-or-later
  */
 
-#include "exm-config.h"
 #include "exm-window.h"
 
 #include "exm-browse-page.h"
-#include "exm-installed-page.h"
+#include "exm-config.h"
 #include "exm-detail-view.h"
-#include "exm-upgrade-assistant.h"
 #include "exm-error-dialog.h"
-
-#include "local/exm-manager.h"
+#include "exm-installed-page.h"
+#include "exm-screenshot-view.h"
+#include "exm-upgrade-assistant.h"
 #include "local/exm-extension.h"
+#include "local/exm-manager.h"
 
 #include <glib/gi18n.h>
 
@@ -43,8 +46,12 @@ struct _ExmWindow
     AdwNavigationView    *navigation_view;
     AdwNavigationPage    *main_view;
     ExmDetailView        *detail_view;
+    ExmScreenshotView    *screenshot_view;
     AdwViewSwitcher      *title;
+    GtkToggleButton      *search_button;
+    GtkSearchBar         *search_bar;
     AdwViewStack         *view_stack;
+    AdwViewStackPage     *installed_stack;
     AdwToastOverlay      *toast_overlay;
     AdwAlertDialog       *remove_dialog;
     AdwAlertDialog       *unsupported_dialog;
@@ -60,14 +67,6 @@ enum {
 
 static GParamSpec *properties [N_PROPS];
 
-static void
-exm_window_finalize (GObject *object)
-{
-    ExmWindow *self = (ExmWindow *)object;
-
-    G_OBJECT_CLASS (exm_window_parent_class)->finalize (object);
-}
-
 static void
 exm_window_get_property (GObject    *object,
                          guint       prop_id,
@@ -86,24 +85,15 @@ exm_window_get_property (GObject    *object,
     }
 }
 
-static void
-exm_window_set_property (GObject      *object,
-                         guint         prop_id,
-                         const GValue *value,
-                         GParamSpec   *pspec)
+GtkSearchBar *
+exm_window_get_search_bar (ExmWindow *self)
 {
-    ExmWindow *self = EXM_WINDOW (object);
-
-    switch (prop_id)
-    {
-    default:
-        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-    }
+  return self->search_bar;
 }
 
 static void
 extension_open_prefs (GtkWidget  *widget,
-                      const char *action_name,
+                      const char *action_name G_GNUC_UNUSED,
                       GVariant   *param)
 {
     ExmWindow *self;
@@ -143,7 +133,7 @@ extension_remove_dialog_response (AdwAlertDialog   *dialog,
 
 static void
 extension_remove (GtkWidget  *widget,
-                  const char *action_name,
+                  const char *action_name G_GNUC_UNUSED,
                   GVariant   *param)
 {
     ExmWindow *self;
@@ -165,7 +155,8 @@ extension_remove (GtkWidget  *widget,
 
 static void
 on_install_done (GObject       *source,
-                 GAsyncResult  *res)
+                 GAsyncResult  *res,
+                 gpointer       user_data G_GNUC_UNUSED)
 {
     GError *error = NULL;
     if (!exm_manager_install_finish (EXM_MANAGER (source), res, &error) && error)
@@ -202,7 +193,7 @@ extension_unsupported_dialog_response (AdwAlertDialog        *dialog,
 
 static void
 extension_install (GtkWidget  *widget,
-                   const char *action_name,
+                   const char *action_name G_GNUC_UNUSED,
                    GVariant   *param)
 {
     ExmWindow *self;
@@ -231,7 +222,7 @@ extension_install (GtkWidget  *widget,
 
 static void
 show_page (GtkWidget  *widget,
-           const char *action_name,
+           const char *action_name G_GNUC_UNUSED,
            GVariant   *param)
 {
     ExmWindow *self;
@@ -271,8 +262,8 @@ show_view (GtkWidget  *widget,
 
 static void
 show_upgrade_assistant (GtkWidget  *widget,
-                        const char *action_name,
-                        GVariant   *param)
+                        const char *action_name G_GNUC_UNUSED,
+                        GVariant   *param G_GNUC_UNUSED)
 {
     ExmWindow *self;
 
@@ -284,7 +275,7 @@ show_upgrade_assistant (GtkWidget  *widget,
 
 static void
 show_error_dialog (GtkWidget  *widget,
-                   const char *action_name,
+                   const char *action_name G_GNUC_UNUSED,
                    GVariant   *param)
 {
     ExmErrorDialog *err_dialog;
@@ -301,7 +292,7 @@ show_error_dialog (GtkWidget  *widget,
 
 static void
 show_error (GtkWidget  *widget,
-            const char *action_name,
+            const char *action_name G_GNUC_UNUSED,
             GVariant   *param)
 {
     ExmWindow *self;
@@ -312,9 +303,8 @@ show_error (GtkWidget  *widget,
 
     g_variant_get (param, "s", &error_text);
 
-    toast = adw_toast_new (_("An error occurred."));
-    adw_toast_set_button_label (toast, _("Details"));
-    adw_toast_set_timeout (toast, 5);
+    toast = adw_toast_new (_("An Error Occurred"));
+    adw_toast_set_button_label (toast, _("_Details"));
 
     adw_toast_set_action_name (toast, "win.show-error-dialog");
     adw_toast_set_action_target (toast, "s", error_text);
@@ -322,23 +312,131 @@ show_error (GtkWidget  *widget,
     adw_toast_overlay_add_toast (self->toast_overlay, toast);
 }
 
+static void
+search_online (GtkWidget  *widget,
+               const char *action_name G_GNUC_UNUSED,
+               GVariant   *param G_GNUC_UNUSED)
+{
+    ExmWindow *self;
+    GtkSearchEntry *search_entry;
+    const char *search_text;
+
+    self = EXM_WINDOW (widget);
+
+    search_entry = exm_browse_page_get_search_entry (self->browse_page);
+    search_text = gtk_editable_get_text (GTK_EDITABLE (gtk_search_bar_get_child (self->search_bar)));
+    adw_view_stack_set_visible_child_name (self->view_stack, "browse");
+    gtk_editable_set_text (GTK_EDITABLE (search_entry) , search_text);
+    gtk_toggle_button_set_active (self->search_button, FALSE);
+}
+
 
 static void
-on_error (ExmManager *manager,
+on_error (ExmManager *manager G_GNUC_UNUSED,
           char       *error_text,
           ExmWindow  *self)
 {
     gtk_widget_activate_action (GTK_WIDGET (self), "win.show-error", "s", error_text);
 }
 
+static void
+on_updates_available (ExmManager *manager G_GNUC_UNUSED,
+                      int         n_updates,
+                      ExmWindow  *self)
+{
+    adw_view_stack_page_set_badge_number (self->installed_stack, (guint) n_updates);
+}
+
+static void
+on_visible_page_changed (AdwViewStack *view_stack,
+                         GtkWidget    *widget G_GNUC_UNUSED,
+                         ExmWindow    *self)
+{
+    gboolean is_installed_page = EXM_IS_INSTALLED_PAGE (adw_view_stack_get_visible_child (view_stack));
+
+    adw_view_stack_page_set_needs_attention (self->installed_stack,
+                                             !is_installed_page &&
+                                             adw_view_stack_page_get_badge_number (self->installed_stack) > 0);
+
+    gtk_widget_set_visible (GTK_WIDGET (self->search_bar),
+                            is_installed_page);
+
+    gtk_widget_set_visible (GTK_WIDGET (self->search_button),
+                            is_installed_page);
+}
+
+void
+on_visible_installed_stack_changed (GObject    *object G_GNUC_UNUSED,
+                                    GParamSpec *pspec G_GNUC_UNUSED,
+                                    gpointer    user_data)
+{
+    ExmWindow *self = (ExmWindow *) user_data;
+    gboolean search_mode;
+    GtkFilterListModel *search_list_model;
+    GtkStack *stack;
+
+    search_mode = gtk_search_bar_get_search_mode (self->search_bar);
+    search_list_model = exm_installed_page_get_search_list_model (self->installed_page);
+    stack = exm_installed_page_get_stack (self->installed_page);
+
+    if (search_mode && g_list_model_get_n_items (G_LIST_MODEL (search_list_model)) > 0)
+        gtk_stack_set_visible_child_name (stack , "page_results");
+    else if (search_mode)
+        gtk_stack_set_visible_child_name (stack , "page_empty");
+    else
+        gtk_stack_set_visible_child_name (stack , "page_list");
+}
+
+static gboolean
+search_open_cb (GtkWidget *widget,
+                GVariant  *args G_GNUC_UNUSED,
+                gpointer   user_data G_GNUC_UNUSED)
+{
+  ExmWindow *self = EXM_WINDOW (widget);
+  AdwNavigationPage *visible_page;
+  const char *visible_stack_name, *visible_page_name;
+
+  visible_page = adw_navigation_view_get_visible_page (self->navigation_view);
+  visible_stack_name = adw_view_stack_get_visible_child_name (self->view_stack);
+  visible_page_name = adw_navigation_page_get_tag (visible_page);
+
+  if (g_strcmp0 (visible_stack_name, "installed") == 0
+      && g_strcmp0 (visible_page_name, "main") == 0)
+  {
+      gboolean is_active = gtk_toggle_button_get_active (self->search_button);
+      gtk_toggle_button_set_active (self->search_button, !is_active);
+  }
+  else if (g_strcmp0 (visible_stack_name, "browse") == 0)
+  {
+      GtkSearchEntry *search_entry = exm_browse_page_get_search_entry (self->browse_page);
+      gtk_widget_grab_focus (GTK_WIDGET (search_entry));
+  }
+  else
+  {
+      return GDK_EVENT_PROPAGATE;
+  }
+
+  return GDK_EVENT_STOP;
+}
+
+static void
+screenshot_zoom (GtkWidget  *widget,
+                 const char *action_name,
+                 GVariant   *parameter G_GNUC_UNUSED)
+{
+  ExmWindow *self = (ExmWindow *)widget;
+
+  g_assert (EXM_IS_WINDOW (self));
+
+  exm_screenshot_view_zoom (self->screenshot_view, action_name);
+}
+
 static void
 exm_window_class_init (ExmWindowClass *klass)
 {
     GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-    object_class->finalize = exm_window_finalize;
     object_class->get_property = exm_window_get_property;
-    object_class->set_property = exm_window_set_property;
 
     properties [PROP_MANAGER]
         = g_param_spec_object ("manager",
@@ -351,19 +449,26 @@ exm_window_class_init (ExmWindowClass *klass)
 
     GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
-    gtk_widget_class_set_template_from_resource (widget_class, "/com/mattjakeman/ExtensionManager/exm-window.ui");
+    gtk_widget_class_set_template_from_resource (widget_class, g_strdup_printf ("%s/exm-window.ui", RESOURCE_PATH));
     gtk_widget_class_bind_template_child (widget_class, ExmWindow, header_bar);
     gtk_widget_class_bind_template_child (widget_class, ExmWindow, installed_page);
     gtk_widget_class_bind_template_child (widget_class, ExmWindow, browse_page);
     gtk_widget_class_bind_template_child (widget_class, ExmWindow, navigation_view);
     gtk_widget_class_bind_template_child (widget_class, ExmWindow, main_view);
     gtk_widget_class_bind_template_child (widget_class, ExmWindow, detail_view);
+    gtk_widget_class_bind_template_child (widget_class, ExmWindow, screenshot_view);
     gtk_widget_class_bind_template_child (widget_class, ExmWindow, title);
+    gtk_widget_class_bind_template_child (widget_class, ExmWindow, search_button);
+    gtk_widget_class_bind_template_child (widget_class, ExmWindow, search_bar);
     gtk_widget_class_bind_template_child (widget_class, ExmWindow, view_stack);
+    gtk_widget_class_bind_template_child (widget_class, ExmWindow, installed_stack);
     gtk_widget_class_bind_template_child (widget_class, ExmWindow, toast_overlay);
     gtk_widget_class_bind_template_child (widget_class, ExmWindow, remove_dialog);
     gtk_widget_class_bind_template_child (widget_class, ExmWindow, unsupported_dialog);
 
+    gtk_widget_class_bind_template_callback (widget_class, on_visible_page_changed);
+    gtk_widget_class_bind_template_callback (widget_class, on_visible_installed_stack_changed);
+
     // TODO: Refactor ExmWindow into a separate ExmController and supply the
     // necessary actions/methods/etc in there. A reference to this new object can
     // then be passed to each page.
@@ -376,27 +481,28 @@ exm_window_class_init (ExmWindowClass *klass)
     gtk_widget_class_install_action (widget_class, "win.show-page", "s", show_page);
     gtk_widget_class_install_action (widget_class, "win.show-error", "s", show_error);
     gtk_widget_class_install_action (widget_class, "win.show-error-dialog", "s", show_error_dialog);
-}
+    gtk_widget_class_install_action (widget_class, "win.search-online", NULL, search_online);
+    gtk_widget_class_install_action (widget_class, "screenshot.zoom-in", NULL, screenshot_zoom);
+    gtk_widget_class_install_action (widget_class, "screenshot.zoom-out", NULL, screenshot_zoom);
+    gtk_widget_class_install_action (widget_class, "screenshot.zoom-reset", NULL, screenshot_zoom);
 
-static void
-do_version_check (ExmWindow *self)
-{
-    GSettings *settings;
-    gchar *version_string;
+    gtk_widget_class_add_binding (widget_class, GDK_KEY_f, GDK_CONTROL_MASK, search_open_cb, NULL);
 
-    settings = g_settings_new (APP_ID);
-    version_string = g_settings_get_string (settings, "last-used-version");
-
-    // In the future, use this to show a toast or notification when
-    // a new version has been installed. Maybe with rich text release notes?
-    // if (strcmp (version_string, APP_VERSION) != 0) { ... }
-
-    g_settings_set_string (settings, "last-used-version", APP_VERSION);
+    gtk_widget_class_add_binding_action (widget_class, GDK_KEY_1, GDK_ALT_MASK, "win.show-page", "s", "installed");
+    gtk_widget_class_add_binding_action (widget_class, GDK_KEY_2, GDK_ALT_MASK, "win.show-page", "s", "browse");
+    gtk_widget_class_add_binding_action (widget_class, GDK_KEY_plus, GDK_CONTROL_MASK, "screenshot.zoom-in", NULL);
+    gtk_widget_class_add_binding_action (widget_class, GDK_KEY_minus, GDK_CONTROL_MASK, "screenshot.zoom-out", NULL);
+    gtk_widget_class_add_binding_action (widget_class, GDK_KEY_0, GDK_CONTROL_MASK, "screenshot.zoom-reset", NULL);
 }
 
 static void
 exm_window_init (ExmWindow *self)
 {
+    g_type_ensure (EXM_TYPE_INSTALLED_PAGE);
+    g_type_ensure (EXM_TYPE_BROWSE_PAGE);
+    g_type_ensure (EXM_TYPE_DETAIL_VIEW);
+    g_type_ensure (EXM_TYPE_SCREENSHOT_VIEW);
+
     gtk_widget_init_template (GTK_WIDGET (self));
 
     if (strstr (APP_ID, ".Devel") != NULL) {
@@ -404,7 +510,7 @@ exm_window_init (ExmWindow *self)
     }
 
     self->manager = exm_manager_new ();
-    g_signal_connect (self->manager, "error-occurred", on_error, self);
+    g_signal_connect (self->manager, "error-occurred", G_CALLBACK (on_error), self);
 
     g_object_set (self->installed_page, "manager", self->manager, NULL);
     g_object_set (self->browse_page, "manager", self->manager, NULL);
@@ -416,6 +522,8 @@ exm_window_init (ExmWindow *self)
                             "shell-version",
                             G_BINDING_SYNC_CREATE);
 
-    // Window must be mapped to show version check dialog
-    g_signal_connect (self, "map", G_CALLBACK (do_version_check), NULL);
+    g_signal_connect (self->manager,
+                      "updates-available",
+                      G_CALLBACK (on_updates_available),
+                      self);
 }
